package com.hand.hls.dp.service.impl;

import cn.hutool.core.date.DateUtil;
import com.hand.hap.core.IRequest;
import com.hand.hls.dp.service.SysDpDownloadLogService;
import com.hand.hls.dp.util.LogUtils;
import org.apache.commons.compress.utils.IOUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.zip.CRC32;
import java.util.zip.CheckedOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author <a href="mailto:zhangnet14@gmail.com">Spencer Chang</a>
 * @date 2020/4/20 10:03
 * @since
 */
@Service
public class SysDpDownloadLogServiceImpl implements SysDpDownloadLogService {
    @Value("${log-path}")
    private String logPath;

    @Override
    public void download(IRequest requestContext, HttpServletRequest request, HttpServletResponse response) throws Exception {
        String zipFileName = DateUtil.format(new Date(),"yyyyMMddHHmmss") + "_logs.zip";
        response.reset();
        response.setHeader("Pragma","No-cache");
        response.setHeader("Cache-Control","no-cache,must-revalidate");
        response.setHeader("Expires","0");
        response.setHeader("Content-Disposition","attachment;filename=" + new String(zipFileName.getBytes(), StandardCharsets.ISO_8859_1));
        response.setContentType("application/x-msdownload;");
        ByteArrayOutputStream byteOutPut = new ByteArrayOutputStream();
        CheckedOutputStream cos = new CheckedOutputStream(byteOutPut,new CRC32());
        ZipOutputStream zos = new ZipOutputStream(cos);
        // 下载7天内的日志
        String[] downloadFiles = new String[7];
        int length = downloadFiles.length;
        // 当前日志
        downloadFiles[0] = logPath + "leaf.log";
        for (int i = 1; i <= length - 1;i++){
            LocalDate tmp = LocalDate.now().minusDays(i);
            Date date = Date.from(tmp.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            downloadFiles[i] = logPath + "leaf-" + DateUtil.format(new Date(),"yyyy-MM-dd") + ".log";
        }
        // 查找日志目录下所有文件
        List<String> files = new ArrayList<>(30);
        LogUtils.findFiles(new File(logPath),files);
        files.forEach(file -> {
            for (int i = 0; i < length; i++) {
                if (file.equals(downloadFiles[i])) {
                    // 创建ZipEntry
                    ZipEntry entry = new ZipEntry(file.substring(logPath.length()));
                    // 放入文件，防止文件名相同的附件报错
                    try {
                        zos.putNextEntry(entry);
                    } catch (Exception e) {
                        // ignore
                    }
                    // 写入文件
                    LogUtils.readFileToZip(file,zos);
                }
            }
        });
        zos.closeEntry();
        zos.close();
        response.setHeader("Content-Length", String.valueOf(byteOutPut.size()));
        IOUtils.copy(new ByteArrayInputStream(byteOutPut.toByteArray()), response.getOutputStream());
        cos.close();
    }
}
