package com.hand.hls.dp.service.impl;

import com.alibaba.druid.sql.SQLUtils;
import com.alibaba.druid.sql.parser.ParserException;
import com.alibaba.druid.util.JdbcConstants;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hand.hap.core.IRequest;
import com.hand.hap.generator.service.impl.DBUtil;
import com.hand.hap.system.dto.ResponseData;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import com.hand.hls.dp.dto.SysDpExecuteHistory;
import com.hand.hls.dp.service.SysDpExecuteHistoryService;
import com.hand.hls.dp.util.SqlConstantUtils;
import com.hand.hls.dp.util.SqlParserCheckUtils;
import com.hand.hls.dp.util.SqlCheckUtils;
import com.hand.hls.exception.HlsCusException;
import hls.core.sys.mapper.SysCodeValueMapper;
import leaf.plugin.export.components.ExcelExportUtil;
import leaf.utils.ConfigUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Future;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @author <a href="mailto:zhangnet14@gmail.com">Spencer Chang</a>
 * @date 2020/4/18 16:07
 * @since
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SysDpExecuteHistoryServiceImpl extends BaseServiceImpl<SysDpExecuteHistory> implements SysDpExecuteHistoryService {
    private static final Logger logger = LoggerFactory.getLogger(SysDpExecuteHistoryServiceImpl.class);
    /**
     * Default initial capacity.
     */
    private static final int DEFAULT_INITIAL_CAPACITY = 16;

    @Autowired
    private SqlSessionFactory sqlSessionFactory;
    @Autowired
    private SysCodeValueMapper sysCodeValueMapper;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;

    /**
     * 查询结果导出excel之前先检验sql是否存在问题?
     *
     * @param requestContext 请求上下文
     * @param ip             请求ip
     * @param dpContext      待执行内容
     */
    @Override
    public void query4ck(IRequest requestContext, String ip, String dpContext) {
        Objects.requireNonNull(dpContext, "传入的语句为空！");
        if (logger.isInfoEnabled()) {
            logger.info("==> start execute query4ck");
            logger.info("ip:[{}] ==> 检验查询待执行语句:[{}]", ip, dpContext);
        }
        final String ipWhiteCheck = sysCodeValueMapper.queryCodeValueMeaning("IP_ALLOW_LIST", "IP");

        if (StringUtils.isNotBlank(dpContext)) {
            StringBuilder msg = new StringBuilder();
            String formatSql = dpContext;
            if (ConfigUtils.isOracle()) {
                try {
                    formatSql = SQLUtils.format(formatSql, JdbcConstants.ORACLE);
                } catch (ParserException ex) {
                    msg.append("语句不正确！错误信息：").append(ex.getMessage());
                }
            } else if (ConfigUtils.isMySQL()) {
                try {
                    formatSql = SQLUtils.format(formatSql, JdbcConstants.MYSQL);
                } catch (ParserException ex) {
                    msg.append("语句不正确！错误信息：").append(ex.getMessage());
                }
            }
            if (StringUtils.isNotEmpty(msg.toString())) {
                throw new IllegalArgumentException(msg.toString());
            }
            String checkMsg = SqlParserCheckUtils.parserCheckSelect(formatSql);
            msg.append(checkMsg);

            // 非内部限制
            if (!StringUtils.startsWith(ip, ipWhiteCheck)) {
                if (SqlCheckUtils.checkSqlIgnoreCase(formatSql, SqlConstantUtils.SQL_STAR)) {
                    msg.append("select语句不允许使用*查询所有字段，请重置语句！");
                }
            }

            if (StringUtils.isNotEmpty(msg.toString())) {
                SysDpExecuteHistory sysDpExecuteHistory = new SysDpExecuteHistory();
                sysDpExecuteHistory.setDehIp(ip);
                sysDpExecuteHistory.setDehUserId(requestContext.getUserId());
                sysDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_SELECT);
                sysDpExecuteHistory.setDehContext(formatSql);
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);
                sysDpExecuteHistory.setDehError(msg.toString());
                self().insertSysDpExecuteHistory(requestContext, sysDpExecuteHistory);
                if (logger.isInfoEnabled()) {
                    logger.info("ip:[{}] ==> 验证不通过:[{}]", ip, msg.toString());
                }
                throw new IllegalArgumentException(msg.toString());
            }
            try (SqlSession sqlSession = sqlSessionFactory.openSession();
                 Connection conn = DBUtil.getConnectionBySqlSession(sqlSession);
                 PreparedStatement preparedStatement = conn.prepareStatement(formatSql);
                 ResultSet rs = preparedStatement.executeQuery()) {
                if (!rs.next()) {
                    throw new IllegalArgumentException("未查询到任何数据！");
                }
            } catch (Exception e) {
                SysDpExecuteHistory sysDpExecuteHistory = new SysDpExecuteHistory();
                sysDpExecuteHistory.setDehIp(ip);
                sysDpExecuteHistory.setDehUserId(requestContext.getUserId());
                sysDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_SELECT);
                sysDpExecuteHistory.setDehContext(formatSql);
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);
                sysDpExecuteHistory.setDehError(e.getMessage());
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                self().insertSysDpExecuteHistory(requestContext, sysDpExecuteHistory);
                if (logger.isWarnEnabled()) {
                    logger.warn("ip:[{}] ==> 出错:[{}]", ip, e.getMessage());
                }
                throw new IllegalArgumentException(msg.toString());
            }
        }
        if (logger.isInfoEnabled()) {
            logger.info("<== end execute query4ck");
        }
    }

    /**
     * 查询结果导出excel
     *
     * @param requestContext 请求上下文
     * @param response       返回
     * @param ip             请求ip
     * @param dpContext      待执行内容
     */
    @Override
    public void query(IRequest requestContext, HttpServletResponse response, String ip, String dpContext) {
        Objects.requireNonNull(dpContext, "传入的语句为空！");
        if (logger.isInfoEnabled()) {
            logger.info("==> start execute query");
            logger.info("ip:[{}] ==> 查询待执行语句:[{}]", ip, dpContext);
        }
        final String ipWhiteCheck = sysCodeValueMapper.queryCodeValueMeaning("IP_ALLOW_LIST", "IP");
        SysDpExecuteHistory sysDpExecuteHistory = new SysDpExecuteHistory();
        sysDpExecuteHistory.setDehIp(ip);
        sysDpExecuteHistory.setDehUserId(requestContext.getUserId());
        sysDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_SELECT);
        sysDpExecuteHistory.setDehContext(dpContext);
        sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);

        JSONArray coumnConfig = new JSONArray(DEFAULT_INITIAL_CAPACITY);
        List<Map<String, Object>> retList = new ArrayList<>(DEFAULT_INITIAL_CAPACITY);
        if (StringUtils.isNotBlank(dpContext)) {
            StringBuilder msg = new StringBuilder();
            String formatSql = dpContext;
            if (ConfigUtils.isOracle()) {
                try {
                    formatSql = SQLUtils.format(formatSql, JdbcConstants.ORACLE);
                } catch (ParserException ex) {
                    msg.append("语句不正确！错误信息：").append(ex.getMessage());
                }
            } else if (ConfigUtils.isMySQL()) {
                try {
                    formatSql = SQLUtils.format(formatSql, JdbcConstants.MYSQL);
                } catch (ParserException ex) {
                    msg.append("语句不正确！错误信息：").append(ex.getMessage());
                }
            }
            if (StringUtils.isNotEmpty(msg.toString())) {
                throw new IllegalArgumentException(msg.toString());
            }
            String checkMsg = SqlParserCheckUtils.parserCheckSelect(formatSql);
            msg.append(checkMsg);

            // 非内部限制
            if (!StringUtils.startsWith(ip, ipWhiteCheck)) {
                if (SqlCheckUtils.checkSqlIgnoreCase(formatSql, SqlConstantUtils.SQL_STAR)) {
                    msg.append("select语句不允许使用*查询所有字段，请重置语句！");
                }
            }
            if (StringUtils.isNotEmpty(msg.toString())) {
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError(msg.toString());
                self().insertSysDpExecuteHistory(requestContext, sysDpExecuteHistory);
                throw new IllegalArgumentException(msg.toString());
            }

            try (SqlSession sqlSession = sqlSessionFactory.openSession();
                 Connection conn = DBUtil.getConnectionBySqlSession(sqlSession);
                 PreparedStatement preparedStatement = conn.prepareStatement(formatSql);
                 ResultSet rs = preparedStatement.executeQuery()) {

                sysDpExecuteHistory.setDehExecuteTime(new Date());

                ResultSetMetaData md = rs.getMetaData();
                int columCount = md.getColumnCount();
                // 组装excel列信息
                for (int i = 1; i <= columCount; i++) {
                    String columnName = md.getColumnName(i);
                    Map<String, Object> columnInfo = new HashMap<>(4);
                    columnInfo.put("width", 150);
                    columnInfo.put("name", columnName);
                    columnInfo.put("align", "left");
                    columnInfo.put("prompt", columnName);
                    JSONObject columnJsonObject = new JSONObject(columnInfo);
                    coumnConfig.add(columnJsonObject);
                }
                while (rs.next()) {
                    Map<String, Object> rowData = new HashMap<>(1);
                    try {
                        getType(rs, md, rowData);
                    } catch (Exception e) {
                        if (logger.isErrorEnabled()) {
                            logger.error(e.getMessage());
                        }
                    }
                    retList.add(rowData);
                }

                sysDpExecuteHistory.setDehRowCount(retList.size());
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DONE);
            } catch (Exception e) {
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError(e.getMessage());
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                self().insertSysDpExecuteHistory(requestContext, sysDpExecuteHistory);
                throw new IllegalArgumentException(e.getMessage());
            }
        }
        self().insertSelective(requestContext, sysDpExecuteHistory);
        // 查询结果导出excel
        ExcelExportUtil excelExportUtil = new ExcelExportUtil();
        excelExportUtil.createExcel(response, coumnConfig, retList);
        if (logger.isInfoEnabled()) {
            logger.info("<== end execute query");
        }
    }

    /**
     * 新增
     *
     * @param requestContext 请求上下文
     * @param ip             请求ip
     * @param dpContext      待执行内容
     * @return 返回消息
     */
    @Override
    public ResponseData insert(IRequest requestContext, String ip, String dpContext) {
        Objects.requireNonNull(dpContext, "传入的语句为空！");
        if (logger.isInfoEnabled()) {
            logger.info("==> start execute insert");
            logger.info("ip:[{}] ==> 新增待执行语句:[{}]", ip, dpContext);
        }
        final String ipWhiteCheck = sysCodeValueMapper.queryCodeValueMeaning("IP_ALLOW_LIST", "IP");
        ResponseData responseData = new ResponseData(false);
        SysDpExecuteHistory sysDpExecuteHistory = new SysDpExecuteHistory();
        sysDpExecuteHistory.setDehIp(ip);
        sysDpExecuteHistory.setDehUserId(requestContext.getUserId());
        sysDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_INSERT);
        sysDpExecuteHistory.setDehContext(dpContext);
        sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);

        if (StringUtils.isNotBlank(dpContext)) {
            StringBuilder msg = new StringBuilder();
            try (SqlSession sqlSession = sqlSessionFactory.openSession();
                 Connection conn = DBUtil.getConnectionBySqlSession(sqlSession)) {
                conn.setAutoCommit(false);
                SysDpExecuteHistory insertDpExecuteHistory = new SysDpExecuteHistory();
                insertDpExecuteHistory.setDehIp(ip);
                insertDpExecuteHistory.setDehUserId(requestContext.getUserId());
                insertDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_INSERT);
                insertDpExecuteHistory.setDehContext(dpContext);
                insertDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);

                // 非内部限制
                if (!StringUtils.startsWith(ip, ipWhiteCheck)) {
                    if (SqlCheckUtils.checkSqlFilterTableIgnoreCase(dpContext, SqlConstantUtils.SQL_FILTER_TABLE)) {
                        msg.append("当前网络环境不允许新增数据！");
                    }
                }
                if (StringUtils.isNotEmpty(msg.toString())) {
                    insertDpExecuteHistory.setDehExecuteTime(new Date());
                    insertDpExecuteHistory.setDehError(msg.toString());
                    self().insertSelective(requestContext, insertDpExecuteHistory);
                    responseData.setMessage("错误消息：" + msg.toString());
                    return responseData;
                }

                try (PreparedStatement preparedStatement = conn.prepareStatement(dpContext)) {
                    int rowCount = preparedStatement.executeUpdate();

                    insertDpExecuteHistory.setDehExecuteTime(new Date());
                    if (logger.isInfoEnabled()) {
                        logger.info("本次插入[{}]条数据。", rowCount);
                    }
                    responseData.setSuccess(rowCount > 0);
                    responseData.setMessage("执行成功，本次插入" + rowCount + "条数据。");
                    insertDpExecuteHistory.setDehRowCount(rowCount);
                    insertDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DONE);
                    self().insertSysDpExecuteHistory(requestContext, insertDpExecuteHistory);
                    conn.commit();
                } catch (SQLException e) {
                    insertDpExecuteHistory.setDehExecuteTime(new Date());
                    insertDpExecuteHistory.setDehError(e.getMessage());
                    insertDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                    self().insertSysDpExecuteHistory(requestContext, insertDpExecuteHistory);
                    responseData.setSuccess(true);
                    responseData.setMessage("错误消息：" + e.getMessage());
                    if (logger.isDebugEnabled()) {
                        logger.debug("ip:[{}] ==> 执行语句:[{}];出错:[{}]", ip, dpContext, e.getMessage());
                    }
                    return responseData;
                }
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError("已执行成功！");
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DONE);
            } catch (SQLException e) {
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError(e.getMessage());
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                responseData.setMessage(e.getMessage());
                if (logger.isWarnEnabled()) {
                    logger.warn("ip:[{}] ==> 出错:[{}]", ip, e.getMessage());
                }
                throw new IllegalArgumentException(e.getMessage());
            }
        }
        self().insertSelective(requestContext, sysDpExecuteHistory);
        if (logger.isInfoEnabled()) {
            logger.info("<== end execute insert");
        }
        return responseData;
    }

    /**
     * 更新
     *
     * @param requestContext 请求上下文
     * @param ip             请求ip
     * @param dpContext      待执行内容
     * @return 返回消息
     */
    @Override
    public ResponseData update(IRequest requestContext, String ip, String dpContext) {
        Objects.requireNonNull(dpContext, "传入的语句为空！");
        if (logger.isInfoEnabled()) {
            logger.info("==> start execute update");
            logger.info("ip:[{}] ==> 更新待执行语句:[{}]", ip, dpContext);
        }
        final String ipWhiteCheck = sysCodeValueMapper.queryCodeValueMeaning("IP_ALLOW_LIST", "IP");
        ResponseData responseData = new ResponseData(false);
        SysDpExecuteHistory sysDpExecuteHistory = new SysDpExecuteHistory();
        sysDpExecuteHistory.setDehIp(ip);
        sysDpExecuteHistory.setDehUserId(requestContext.getUserId());
        sysDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_UPDATE);
        sysDpExecuteHistory.setDehContext(dpContext);
        sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);

        if (StringUtils.isNotBlank(dpContext)) {
            StringBuilder msg = new StringBuilder();
            try (SqlSession sqlSession = sqlSessionFactory.openSession();
                 Connection conn = DBUtil.getConnectionBySqlSession(sqlSession)) {
                conn.setAutoCommit(false);
                SysDpExecuteHistory updateDpExecuteHistory = new SysDpExecuteHistory();
                updateDpExecuteHistory.setDehIp(ip);
                updateDpExecuteHistory.setDehUserId(requestContext.getUserId());
                updateDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_UPDATE);
                updateDpExecuteHistory.setDehContext(dpContext);
                updateDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);

                // 非内部限制
                if (!StringUtils.startsWith(ip, ipWhiteCheck)) {
                    if (SqlCheckUtils.checkSqlFilterTableIgnoreCase(dpContext, SqlConstantUtils.SQL_FILTER_TABLE)) {
                        msg.append("当前网络环境不允许修改数据！");
                    }
                }
                if (StringUtils.isNotEmpty(msg.toString())) {
                    updateDpExecuteHistory.setDehExecuteTime(new Date());
                    updateDpExecuteHistory.setDehError(msg.toString());
                    self().insertSelective(requestContext, updateDpExecuteHistory);
                    responseData.setMessage("错误消息：" + msg.toString());
                    return responseData;
                }

                String sql = SqlConstantUtils.SELECT_COUNT;
                // 默认截取update之后语句
                if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_SET)) {
                    sql = sql + StringUtils.substring(dpContext, 6, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_SET));
                } else if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_SET_UPPER)) {
                    sql = sql + StringUtils.substring(dpContext, 6, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_SET_UPPER));
                }
                if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_WHERE)) {
                    sql = sql + StringUtils.substring(dpContext, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_WHERE));
                } else if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_WHERE_UPPER)) {
                    sql = sql + StringUtils.substring(dpContext, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_WHERE_UPPER));
                }

                // 先查询受where条件影响数据
                int rowCount = 0;
                try (PreparedStatement preparedStatementQuery = conn.prepareStatement(sql);
                     ResultSet rs = preparedStatementQuery.executeQuery()) {
                    while (rs.next()) {
                        rowCount = rs.getInt("TOTALCOUNT");
                    }
                    if (logger.isInfoEnabled()) {
                        logger.info("==> 执行sql[{}],本次更新条件查询：[{}]条数据。", sql, rowCount);
                    }
                } catch (SQLException e) {
                    updateDpExecuteHistory.setDehExecuteTime(new Date());
                    updateDpExecuteHistory.setDehContext(sql);
                    updateDpExecuteHistory.setDehError(e.getMessage());
                    updateDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                    self().insertSysDpExecuteHistory(requestContext, updateDpExecuteHistory);
                    responseData.setSuccess(true);
                    responseData.setMessage("错误消息：" + e.getMessage());
                    if (logger.isDebugEnabled()) {
                        logger.debug("ip:[{}] ==> 执行语句:[{}];出错:[{}]", ip, sql, e.getMessage());
                    }
                    return responseData;
                }
                // 备份数据
                String bakSql = SqlConstantUtils.SELECT_FROM;
                // 默认截取update之后语句
                if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_SET)) {
                    bakSql = bakSql + StringUtils.substring(dpContext, 6, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_SET));
                } else if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_SET_UPPER)) {
                    bakSql = bakSql + StringUtils.substring(dpContext, 6, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_SET_UPPER));
                }
                if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_WHERE)) {
                    bakSql = bakSql + StringUtils.substring(dpContext, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_WHERE));
                } else if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_WHERE_UPPER)) {
                    bakSql = bakSql + StringUtils.substring(dpContext, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_WHERE_UPPER));
                }
                String bakData = bakData(conn, bakSql);

                // 执行传入语句
                try (PreparedStatement preparedStatement = conn.prepareStatement(dpContext)) {
                    int updateRowCount = preparedStatement.executeUpdate();
                    if (logger.isInfoEnabled()) {
                        logger.info("==> 执行sql[{}],本次更新：[{}]条数据。", dpContext, updateRowCount);
                    }
                    updateDpExecuteHistory.setDehExecuteTime(new Date());

                    // 查询结果==更新结果
                    if (updateRowCount == rowCount) {
                        conn.commit();
                        if (logger.isInfoEnabled()) {
                            logger.info("本次更新[{}]条数据。", updateRowCount);
                        }
                        responseData.setSuccess(updateRowCount > 0);
                        responseData.setMessage("执行成功，本次更新" + updateRowCount + "条数据。");
                        responseData.setTotal((long) updateRowCount);
                        updateDpExecuteHistory.setDehRowCount(updateRowCount);
                        updateDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DONE);
                        updateDpExecuteHistory.setDehBakData(bakData);
                        self().insertSysDpExecuteHistory(requestContext, updateDpExecuteHistory);
                    } else {
                        conn.rollback();
                        updateDpExecuteHistory.setDehError("更新条目与查询条目不一致！本次更新前条件查询" + rowCount + "条数据。本次更新" + updateRowCount + "条数据。");
                        updateDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                        self().insertSysDpExecuteHistory(requestContext, updateDpExecuteHistory);
                        responseData.setSuccess(true);
                        responseData.setMessage("错误消息：：更新条目与查询条目不一致！本次更新前条件查询" + rowCount + "条数据。本次更新" + updateRowCount + "条数据。");
                        return responseData;
                    }
                } catch (SQLException e) {
                    updateDpExecuteHistory.setDehExecuteTime(new Date());
                    updateDpExecuteHistory.setDehError(e.getMessage());
                    updateDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                    self().insertSysDpExecuteHistory(requestContext, updateDpExecuteHistory);
                    responseData.setSuccess(true);
                    responseData.setMessage("错误消息：" + e.getMessage());
                    if (logger.isDebugEnabled()) {
                        logger.debug("ip:[{}] ==> 执行语句:[{}];出错:[{}]", ip, dpContext, e.getMessage());
                    }
                    return responseData;
                }
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError("已执行成功！");
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DONE);
            } catch (SQLException e) {
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError(e.getMessage());
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                responseData.setMessage(e.getMessage());
                self().insertSysDpExecuteHistory(requestContext, sysDpExecuteHistory);
                if (logger.isWarnEnabled()) {
                    logger.warn("ip:[{}] ==> 出错:[{}]", ip, e.getMessage());
                }
                throw new IllegalArgumentException(e.getMessage());
            }
        }
        self().insertSelective(requestContext, sysDpExecuteHistory);
        if (logger.isInfoEnabled()) {
            logger.info("<== end execute update");
        }
        return responseData;
    }

    /**
     * 删除
     *
     * @param requestContext 请求上下文
     * @param ip             请求ip
     * @param dpContext      待执行内容
     * @return 返回消息
     */
    @Override
    public ResponseData delete(IRequest requestContext, String ip, String dpContext) {
        Objects.requireNonNull(dpContext, "传入的语句为空！");
        if (logger.isInfoEnabled()) {
            logger.info("==> start execute delete");
            logger.info("ip:[{}] ==> 删除待执行语句:[{}]", ip, dpContext);
        }
        final String ipWhiteCheck = sysCodeValueMapper.queryCodeValueMeaning("IP_ALLOW_LIST", "IP");
        ResponseData responseData = new ResponseData(false);
        SysDpExecuteHistory sysDpExecuteHistory = new SysDpExecuteHistory();
        sysDpExecuteHistory.setDehIp(ip);
        sysDpExecuteHistory.setDehUserId(requestContext.getUserId());
        sysDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_DELETE);
        sysDpExecuteHistory.setDehContext(dpContext);
        sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);

        if (StringUtils.isNotBlank(dpContext)) {
            StringBuilder msg = new StringBuilder();
            try (SqlSession sqlSession = sqlSessionFactory.openSession();
                 Connection conn = DBUtil.getConnectionBySqlSession(sqlSession)) {
                conn.setAutoCommit(false);
                SysDpExecuteHistory delDpExecuteHistory = new SysDpExecuteHistory();
                delDpExecuteHistory.setDehIp(ip);
                delDpExecuteHistory.setDehUserId(requestContext.getUserId());
                delDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_DELETE);
                delDpExecuteHistory.setDehContext(dpContext);
                delDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);

                // 非内部限制
                if (!StringUtils.startsWith(ip, ipWhiteCheck)) {
                    if (SqlCheckUtils.checkSqlFilterTableIgnoreCase(dpContext, SqlConstantUtils.SQL_FILTER_TABLE)) {
                        msg.append("当前网络环境不允许删除数据！");
                    }
                }
                if (StringUtils.isNotEmpty(msg.toString())) {
                    delDpExecuteHistory.setDehExecuteTime(new Date());
                    delDpExecuteHistory.setDehError(msg.toString());
                    self().insertSelective(requestContext, delDpExecuteHistory);
                    responseData.setMessage("错误消息：" + msg.toString());
                    return responseData;
                }

                String sql = SqlConstantUtils.SELECT_COUNT;
                // 默认截取from之后语句
                if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_FROM)) {
                    sql = sql + StringUtils.substring(dpContext, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_FROM));
                } else if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_FROM_UPPER)) {
                    sql = sql + StringUtils.substring(dpContext, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_FROM_UPPER));
                }

                // 先查询受where条件影响数据
                int rowCount = 0;
                try (PreparedStatement preparedStatementQuery = conn.prepareStatement(sql);
                     ResultSet rs = preparedStatementQuery.executeQuery()) {
                    while (rs.next()) {
                        rowCount = rs.getInt("TOTALCOUNT");
                    }
                    if (logger.isInfoEnabled()) {
                        logger.info("==> 执行sql[{}],本次删除前条件查询：[{}]条数据。", sql, rowCount);
                    }
                } catch (SQLException e) {
                    sysDpExecuteHistory.setDehExecuteTime(new Date());
                    delDpExecuteHistory.setDehContext(sql);
                    sysDpExecuteHistory.setDehError(e.getMessage());
                    sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                    self().insertSysDpExecuteHistory(requestContext, delDpExecuteHistory);
                    responseData.setSuccess(true);
                    responseData.setMessage("错误消息：" + e.getMessage());
                    if (logger.isDebugEnabled()) {
                        logger.debug("ip:[{}] ==> 执行语句:[{}];出错:[{}]", ip, sql, e.getMessage());
                    }
                    return responseData;
                }
                // 备份数据
                String bakSql = "SELECT * ";
                // 默认截取from之后语句
                if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_FROM)) {
                    bakSql = bakSql + StringUtils.substring(dpContext, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_FROM));
                } else if (StringUtils.contains(dpContext, SqlConstantUtils.SQL_FROM_UPPER)) {
                    bakSql = bakSql + StringUtils.substring(dpContext, StringUtils.indexOf(dpContext, SqlConstantUtils.SQL_FROM_UPPER));
                }
                String bakData = bakData(conn, bakSql);

                // 执行传入语句
                try (PreparedStatement preparedStatement = conn.prepareStatement(dpContext)) {
                    int deleteRowCount = preparedStatement.executeUpdate();
                    if (logger.isInfoEnabled()) {
                        logger.info("==> 执行sql[{}],本次删除：[{}]条数据。", dpContext, deleteRowCount);
                    }
                    delDpExecuteHistory.setDehExecuteTime(new Date());

                    // 查询结果==删除结果
                    if (deleteRowCount == rowCount) {
                        conn.commit();
                        if (logger.isInfoEnabled()) {
                            logger.info("本次删除[{}]条数据。", rowCount);
                        }
                        responseData.setSuccess(deleteRowCount > 0);
                        responseData.setMessage("执行成功，本次删除" + deleteRowCount + "条数据。");
                        responseData.setTotal((long) deleteRowCount);
                        delDpExecuteHistory.setDehRowCount(deleteRowCount);
                        delDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DONE);
                        delDpExecuteHistory.setDehBakData(bakData);
                        self().insertSysDpExecuteHistory(requestContext, delDpExecuteHistory);
                    } else {
                        conn.rollback();
                        delDpExecuteHistory.setDehError("删除条目与查询条目不一致！本次删除前条件查询" + rowCount + "条数据。本次删除" + deleteRowCount + "条数据。");
                        delDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                        self().insertSysDpExecuteHistory(requestContext, delDpExecuteHistory);
                        responseData.setSuccess(true);
                        responseData.setMessage("错误消息：删除条目与查询条目不一致！本次删除前条件查询" + rowCount + "条数据。本次删除" + deleteRowCount + "条数据。");
                        return responseData;
                    }
                } catch (SQLException e) {
                    sysDpExecuteHistory.setDehExecuteTime(new Date());
                    sysDpExecuteHistory.setDehError(e.getMessage());
                    sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                    self().insertSysDpExecuteHistory(requestContext, delDpExecuteHistory);
                    responseData.setSuccess(true);
                    responseData.setMessage("错误消息：" + e.getMessage());
                    if (logger.isDebugEnabled()) {
                        logger.debug("ip:[{}] ==> 执行语句:[{}];出错:[{}]", ip, dpContext, e.getMessage());
                    }
                    return responseData;
                }
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError("已执行成功！");
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DONE);
            } catch (Exception e) {
                sysDpExecuteHistory.setDehError(e.getMessage());
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                self().insertSysDpExecuteHistory(requestContext, sysDpExecuteHistory);
                if (logger.isWarnEnabled()) {
                    logger.warn("ip:[{}] ==> 出错:[{}]", ip, e.getMessage());
                }
                throw new IllegalArgumentException(e.getMessage());
            }
        }
        self().insertSelective(requestContext, sysDpExecuteHistory);
        if (logger.isInfoEnabled()) {
            logger.info("<== end execute delete");
        }
        return responseData;
    }

    /**
     * ddl
     *
     * @param requestContext 请求上下文
     * @param ip             请求ip
     * @param dpContext      待执行内容
     * @return 返回消息
     */
    @Override
    public ResponseData ddl(IRequest requestContext, String ip, String dpContext) {
        Objects.requireNonNull(dpContext, "传入的语句为空！");
        if (logger.isInfoEnabled()) {
            logger.info("==> start execute ddl");
            logger.info("ip:[{}] ==> 待执行DDL语句:[{}]", ip, dpContext);
        }
        final String ipWhiteCheck = sysCodeValueMapper.queryCodeValueMeaning("IP_ALLOW_LIST", "IP");
        ResponseData responseData = new ResponseData(false);
        SysDpExecuteHistory sysDpExecuteHistory = new SysDpExecuteHistory();
        sysDpExecuteHistory.setDehIp(ip);
        sysDpExecuteHistory.setDehUserId(requestContext.getUserId());
        sysDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_DDL);
        sysDpExecuteHistory.setDehContext(dpContext);
        sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_UNDO);

        if (StringUtils.isNotBlank(dpContext)) {
            StringBuilder msg = new StringBuilder();
            // 非内部限制
            if (!StringUtils.startsWith(ip, ipWhiteCheck)) {
                msg.append("当前网络环境不允许执行语句！");
            }

            if (StringUtils.isNotEmpty(msg.toString())) {
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError(msg.toString());
                self().insertSelective(requestContext, sysDpExecuteHistory);
                responseData.setMessage(msg.toString());
                return responseData;
            }
            try (SqlSession sqlSession = sqlSessionFactory.openSession();
                 Connection conn = DBUtil.getConnectionBySqlSession(sqlSession)) {
                SysDpExecuteHistory ddlDpExecuteHistory = new SysDpExecuteHistory();
                ddlDpExecuteHistory.setDehIp(ip);
                ddlDpExecuteHistory.setDehUserId(requestContext.getUserId());
                ddlDpExecuteHistory.setDehExecuteType(SqlConstantUtils.SQL_EXEC_DDL);
                ddlDpExecuteHistory.setDehContext(dpContext);

                if (StringUtils.isNotEmpty(msg.toString())) {
                    ddlDpExecuteHistory.setDehExecuteTime(new Date());
                    ddlDpExecuteHistory.setDehError(msg.toString());
                    self().insertSelective(requestContext, ddlDpExecuteHistory);
                    responseData.setMessage("错误消息：" + msg.toString());
                    return responseData;
                }
                try (PreparedStatement ddl = conn.prepareStatement(dpContext)) {
                    ddl.executeUpdate();
                    ddlDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DONE);
                    ddlDpExecuteHistory.setDehExecuteTime(new Date());
                    self().insertSysDpExecuteHistory(requestContext, ddlDpExecuteHistory);
                } catch (SQLException e) {
                    sysDpExecuteHistory.setDehExecuteTime(new Date());
                    sysDpExecuteHistory.setDehError(e.getMessage());
                    sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                    self().insertSysDpExecuteHistory(requestContext, ddlDpExecuteHistory);
                    responseData.setSuccess(true);
                    responseData.setMessage("错误消息：" + e.getMessage());
                    if (logger.isDebugEnabled()) {
                        logger.debug("ip:[{}] ==> 执行语句:[{}];出错:[{}]", ip, dpContext, e.getMessage());
                    }
                    return responseData;
                }
                responseData.setSuccess(true);
                responseData.setMessage("执行成功！");
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError("已执行成功！");
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DONE);
            } catch (Exception e) {
                sysDpExecuteHistory.setDehExecuteTime(new Date());
                sysDpExecuteHistory.setDehError(e.getMessage());
                sysDpExecuteHistory.setDehStatus(SqlConstantUtils.SQL_EXEC_DE);
                self().insertSysDpExecuteHistory(requestContext, sysDpExecuteHistory);
                if (logger.isWarnEnabled()) {
                    logger.warn("ip:[{}] ==> 出错:[{}]", ip, e.getMessage());
                }
                throw new IllegalArgumentException(e.getMessage());
            }
        }
        self().insertSelective(requestContext, sysDpExecuteHistory);
        if (logger.isInfoEnabled()) {
            logger.info("<== end execute ddl");
        }
        return responseData;
    }

    /**
     * 备份数据
     *
     * @param conn   链接
     * @param bakSql 备份待执行sql
     * @return 返回消息
     * @throws SQLException sql例外
     */
    private String bakData(Connection conn, String bakSql) throws SQLException {
        PreparedStatement bakQuery = conn.prepareStatement(bakSql);
        ResultSet bakRs = bakQuery.executeQuery();
        String bakData = null;
        try {
            bakData = convertToJson(bakRs);
        } catch (Exception e) {
            if (logger.isDebugEnabled()) {
                logger.debug("==> 执行备份语句:[{}];出错:[{}]", bakSql, e.getMessage());
            }
        }
        bakQuery.close();
        bakRs.close();
        return bakData;
    }

    /**
     * rs结果转换jsoc
     *
     * @param rs JDBC返回结果集
     * @return 返回消息
     * @throws Exception 例外
     */
    private static String convertToJson(ResultSet rs) throws Exception {
        JSONArray jsonArray = new JSONArray(DEFAULT_INITIAL_CAPACITY);
        ResultSetMetaData metaData = rs.getMetaData();

        while (rs.next()) {
            JSONObject jsonObject = new JSONObject(DEFAULT_INITIAL_CAPACITY);
            getType(rs, metaData, jsonObject);
            jsonArray.add(jsonObject);
        }
        return jsonArray.toJSONString();
    }

    /**
     * 根据类型转换数据
     *
     * @param rs         JDBC返回结果集
     * @param md         元数据
     * @param jsonObject json对象
     * @throws Exception 例外
     */
    private static void getType(ResultSet rs, ResultSetMetaData md, Map<String, Object> jsonObject) throws Exception {
        int columnCount = md.getColumnCount();
        for (int i = 1; i <= columnCount; i++) {
            String columnName = md.getColumnName(i);
            int columnType = md.getColumnType(i);
            switch (columnType) {
                case Types.ARRAY:
                    jsonObject.put(columnName, rs.getArray(columnName));
                    break;
                case Types.BIGINT:
                    jsonObject.put(columnName, rs.getInt(columnName));
                    break;
                case Types.BOOLEAN:
                    jsonObject.put(columnName, rs.getBoolean(columnName));
                    break;
                case Types.BLOB:
                    Blob blob = rs.getBlob(columnName);
                    byte[] bytes = new byte[(int) blob.length()];
                    try (InputStream in = blob.getBinaryStream()) {
                        in.read(bytes);
                        String tmp = new String(bytes);
                        jsonObject.put(columnName, tmp);
                    } catch (IOException e) {
                        // ignore
                    }
                    break;
                case Types.DOUBLE:
                    jsonObject.put(columnName, rs.getDouble(columnName));
                    break;
                case Types.FLOAT:
                    jsonObject.put(columnName, rs.getFloat(columnName));
                    break;
                case Types.INTEGER:
                    jsonObject.put(columnName, rs.getInt(columnName));
                    break;
                case Types.NVARCHAR:
                    jsonObject.put(columnName, rs.getNString(columnName));
                    break;
                case Types.VARCHAR:
                    jsonObject.put(columnName, rs.getString(columnName));
                    break;
                case Types.TINYINT:
                    jsonObject.put(columnName, rs.getInt(columnName));
                    break;
                case Types.SMALLINT:
                    jsonObject.put(columnName, rs.getInt(columnName));
                    break;
                case Types.DATE:
                    jsonObject.put(columnName, rs.getDate(columnName));
                    break;
                case Types.TIMESTAMP:
                    jsonObject.put(columnName, rs.getTimestamp(columnName));
                    break;
                case Types.CLOB:
                    jsonObject.put(columnName, rs.getString(columnName));
                    break;
                case Types.DECIMAL:
                    jsonObject.put(columnName, rs.getBigDecimal(columnName));
                    break;
                case Types.NUMERIC:
                    jsonObject.put(columnName, rs.getBigDecimal(columnName));
                    break;
                default:
                    jsonObject.put(columnName, rs.getObject(columnName));
                    break;
            }
        }
    }

    /**
     * 自定义事务，保存查询预校验执行记录
     *
     * @param requestContext      请求上下文
     * @param sysDpExecuteHistory 传入的参数
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    @Override
    public void insertSysDpExecuteHistory(IRequest requestContext, SysDpExecuteHistory sysDpExecuteHistory) {
        self().insertSelective(requestContext, sysDpExecuteHistory);
    }

    @Override
    public ResponseData execute(IRequest requestContext, String ip, String dpContext) throws Exception {
        ResponseData responseData = new ResponseData(true);
        if (StringUtils.isNotBlank(dpContext)) {
            StringBuilder msg = new StringBuilder();
            // 支持多条语句执行，以“;“隔开
            String[] undos = StringUtils.trimToEmpty(dpContext).split(";");
            int length = undos.length;
            ConcurrentHashMap<Integer, String> retMsg = null;
            if (length >= 1) {
                retMsg = new ConcurrentHashMap<>(1);
            }
            List<String> result = new ArrayList<>(retMsg.size());
            AtomicInteger atomicInteger = new AtomicInteger();
            for (int i = 1; i <= length; i++) {
                String undo = undos[i-1];
                if (StringUtils.isBlank(undo)) {
                    continue;
                }
                String formatSql = undo;
                if (ConfigUtils.isOracle()) {
                    try {
                        formatSql = SQLUtils.format(formatSql, JdbcConstants.ORACLE);
                    } catch (ParserException ex) {
                        msg.append("语句不正确！错误信息：").append(ex.getMessage());
                    }
                } else if (ConfigUtils.isMySQL()) {
                    try {
                        formatSql = SQLUtils.format(formatSql, JdbcConstants.MYSQL);
                    } catch (ParserException ex) {
                        msg.append("语句不正确！错误信息：").append(ex.getMessage());
                    }
                }
                if (StringUtils.isNotEmpty(msg.toString())) {
                    throw new HlsCusException(msg.toString());
                }
                // 判断语句开始关键字
                if (StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.SQL_EXEC_SELECT)) {
                    throw new HlsCusException("执行按钮不支持SELECT查询语句，请使用查询按钮");
                } else if (StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.SQL_EXEC_INSERT)) {
                    String checkMsg = SqlParserCheckUtils.parserCheckInsert(formatSql);
                    msg.append(checkMsg);
                    if (StringUtils.isNotEmpty(msg.toString())) {
                        throw new HlsCusException(msg.toString());
                    }
                    Future<?> future = taskExecutor.submit(new SubThreadTask(self(), requestContext, ip, formatSql, SqlConstantUtils.SQL_EXEC_INSERT, length, atomicInteger, i, retMsg));
                    while (true) {
                        if (future.isDone()) {
                            retMsg.forEach((k, v) -> result.add("第" + k + "行语句，" + v));
                            retMsg.clear();
                            break;
                        }
                    }
                } else if (StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.SQL_EXEC_UPDATE)) {
                    String checkMsg = SqlParserCheckUtils.parserCheckUpdate(formatSql);
                    msg.append(checkMsg);
                    if (StringUtils.isNotEmpty(msg.toString())) {
                        throw new HlsCusException(msg.toString());
                    }
                    Future<?> future = taskExecutor.submit(new SubThreadTask(self(), requestContext, ip, formatSql, SqlConstantUtils.SQL_EXEC_UPDATE, length, atomicInteger, i, retMsg));
                    while (true) {
                        if (future.isDone()) {
                            retMsg.forEach((k, v) -> result.add("第" + k + "行语句，" + v));
                            retMsg.clear();
                            break;
                        }
                    }
                } else if (StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.SQL_EXEC_DELETE)) {
                    String checkMsg = SqlParserCheckUtils.parserCheckDelete(formatSql);
                    msg.append(checkMsg);
                    if (StringUtils.isNotEmpty(msg.toString())) {
                        throw new HlsCusException(msg.toString());
                    }
                    Future<?> future = taskExecutor.submit(new SubThreadTask(self(), requestContext, ip, formatSql, SqlConstantUtils.SQL_EXEC_DELETE, length, atomicInteger, i, retMsg));
                    while (true) {
                        if (future.isDone()) {
                            retMsg.forEach((k, v) -> result.add("第" + k + "行语句，" + v));
                            retMsg.clear();
                            break;
                        }
                    }
                } else if (StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.DDL_CREATE_TABLE) ||
                        StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.DDL_ALTER_TABLE) ||
                        StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.DDL_COMMENT_TABLE) ||
                        StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.DDL_COMMENT_ON_COLUMN) ||
                        StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.DDL_CREATE_INDEX) ||
                        StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.DDL_CREATE_SEQUENCE) ||
                        StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.DDL_CREATE_SEQUENCE) ||
                        StringUtils.startsWithIgnoreCase(formatSql, SqlConstantUtils.DDL_ALTER_SYSTEM_KILL_SESSION)) {
                    String checkMsg = SqlParserCheckUtils.parserCheckDDL(formatSql);
                    msg.append(checkMsg);
                    if (StringUtils.isNotEmpty(msg.toString())) {
                        throw new HlsCusException(msg.toString());
                    }
                    Future<?> future = taskExecutor.submit(new SubThreadTask(self(), requestContext, ip, formatSql, SqlConstantUtils.SQL_EXEC_DDL, length, atomicInteger, i, retMsg));
                    while (true) {
                        if (future.isDone()) {
                            retMsg.forEach((k, v) -> result.add("第" + k + "行语句，" + v));
                            retMsg.clear();
                            break;
                        }
                    }
                }
            }
            if (!result.isEmpty()) {
                responseData.setRows(result);
            }
        }
        return responseData;
    }

    private static class SubThreadTask implements Runnable {
        private final ReentrantLock mainLock = new ReentrantLock();
        private final SysDpExecuteHistoryService service;
        private final IRequest requestContext;
        private final String sql;
        private final String ip;
        private final String execType;
        private final Integer count;
        private final AtomicInteger atomicInteger;
        private final Integer seq;
        private final ConcurrentHashMap<Integer, String> retMsg;

        private SubThreadTask(SysDpExecuteHistoryService service, IRequest requestContext, String ip, String sql,
                              String execType, Integer count, AtomicInteger atomicInteger, Integer seq, ConcurrentHashMap<Integer, String> retMsg) {
            this.service = service;
            this.requestContext = requestContext;
            this.ip = ip;
            this.sql = sql;
            this.execType = execType;
            this.count = count;
            this.atomicInteger = atomicInteger;
            this.seq = seq;
            this.retMsg = retMsg;
        }

        @Override
        public void run() {
            final ReentrantLock mainLock = this.mainLock;
            mainLock.lock();
            try {
                String threadName = Thread.currentThread().getName();
                ResponseData responseData = new ResponseData();
                int index = atomicInteger.getAndIncrement();
                if (StringUtils.equals(execType, SqlConstantUtils.SQL_EXEC_INSERT)) {
                    responseData = service.insert(requestContext, ip, sql);
                } else if (StringUtils.equals(execType, SqlConstantUtils.SQL_EXEC_UPDATE)) {
                    responseData = service.update(requestContext, ip, sql);
                } else if (StringUtils.equals(execType, SqlConstantUtils.SQL_EXEC_DELETE)) {
                    responseData = service.delete(requestContext, ip, sql);
                } else if (StringUtils.equals(execType, SqlConstantUtils.SQL_EXEC_DDL)) {
                    responseData = service.ddl(requestContext, ip, sql);
                }
                logger.info("==> [{}]-->第[{}]次执行：[{}]", threadName, index, sql);
                retMsg.put(seq, responseData.getMessage());
                logger.info("==> 执行语句[{}]，详细信息：[{}]", sql, responseData.getMessage());
                if (index == count - 1 || count == 1) {
                    logger.info("<== ALL DONE");
                }
            } catch (Exception e) {
                retMsg.put(seq, e.getMessage());
                logger.info("==> 执行语句[{}]出错，详细信息：[{}]", sql, e.getMessage());
            } finally {
                mainLock.unlock();
            }
        }
    }
}