package com.hand.hls.dp.util;

import com.alibaba.druid.sql.ast.SQLExpr;
import com.alibaba.druid.sql.ast.SQLObject;
import com.alibaba.druid.sql.ast.SQLStatement;
import com.alibaba.druid.sql.ast.expr.SQLBinaryOpExpr;
import com.alibaba.druid.sql.ast.expr.SQLBinaryOperator;
import com.alibaba.druid.sql.ast.expr.SQLQueryExpr;
import com.alibaba.druid.sql.ast.statement.*;
import com.alibaba.druid.sql.dialect.oracle.parser.OracleStatementParser;
import org.apache.commons.collections.CollectionUtils;

import java.util.List;
import java.util.Objects;

/**
 * @author <a href="mailto:zhangnet14@gmail.com">Spencer Chang</a>
 * @date 2020/6/10 15:09
 * @since 1.0
 */
public final class SqlParserCheckUtils {
    private static final int STMTLISTSIZE = 2;
    /**
     * 校验查询语句
     *
     * @param sql 待校验语句
     * @return 返回信息
     */
    public static String parserCheckSelect(String sql) {
        StringBuilder ret = new StringBuilder();
        OracleStatementParser parser = new OracleStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();
        if (Objects.isNull(stmtList) || CollectionUtils.isEmpty(stmtList) || CollectionUtils.size(stmtList) >= STMTLISTSIZE) {
            ret.append("语句不正确！");
            return ret.toString();
        }
        for (SQLStatement stmt : stmtList) {
            if (stmt instanceof SQLSelectStatement) {
                SQLSelectStatement selectStatement = (SQLSelectStatement) stmt;
                SQLSelect sqlSelect = selectStatement.getSelect();
                SQLSelectQueryBlock queryBlock = sqlSelect.getQueryBlock();
                String selectRet = parseSelectBlock(ret, queryBlock);
                if (selectRet != null) {return selectRet;}
                // 校验查询字段里的SUB SELECT
                List<SQLSelectItem> sqlSelectItemList = queryBlock.getSelectList();
                for (SQLSelectItem selectItem : sqlSelectItemList) {
                    SQLExpr itemExpr = selectItem.getExpr();
                    if (itemExpr instanceof SQLQueryExpr) {
                        SQLQueryExpr itemQueryExpr = (SQLQueryExpr) itemExpr;
                        SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                        SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                        String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                        if (subSelectRet != null) {return subSelectRet;}
                    }
                }
            } else {
                ret.append("不是SELECT语句！");
            }
        }
        return ret.toString();
    }

    private static String parseSelectBlock(StringBuilder ret, SQLSelectQueryBlock queryBlock) {
        SQLExpr where = queryBlock.getWhere();
        if (Objects.isNull(where)) {
            return ret.append("SELECT语句没有WHERE条件！").toString();
        }
        if (where instanceof SQLBinaryOpExpr) {
            SQLBinaryOpExpr sqlBinaryOpExpr = (SQLBinaryOpExpr) where;
            SQLBinaryOperator sqlBinaryOperator = sqlBinaryOpExpr.getOperator();
            if (Objects.equals(SQLBinaryOperator.Equality, sqlBinaryOperator)) {
                SQLExpr right = sqlBinaryOpExpr.getRight();
                if (right instanceof SQLQueryExpr) {
                    SQLQueryExpr itemQueryExpr = (SQLQueryExpr) right;
                    SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                    SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                    parseSelectBlock(ret, subQueryBlock);
                    if (ret != null) {
                        return ret.toString();
                    }
                }
                SQLExpr left = sqlBinaryOpExpr.getLeft();
                if (left instanceof SQLQueryExpr) {
                    SQLQueryExpr itemQueryExpr = (SQLQueryExpr) left;
                    SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                    SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                    String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                    if (subSelectRet != null) {
                        return subSelectRet;
                    }
                }
                if (Objects.equals(right, left)) {
                    assert ret != null;
                    return ret.append("SELECT语句WHERE条件含有1 = 1！").toString();
                }
            } else {
                // WHERE条件集合
                List<SQLObject> whereList = where.getChildren();
                for (SQLObject so : whereList) {
                    if (so instanceof SQLBinaryOpExpr) {
                        SQLBinaryOpExpr sbo = (SQLBinaryOpExpr) so;
                        SQLExpr right = sbo.getRight();
                        if (right instanceof SQLQueryExpr) {
                            SQLQueryExpr itemQueryExpr = (SQLQueryExpr) right;
                            SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                            SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                            String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                            if (subSelectRet != null) {
                                return subSelectRet;
                            }
                        }
                        SQLExpr left = sbo.getLeft();
                        if (left instanceof SQLQueryExpr) {
                            SQLQueryExpr itemQueryExpr = (SQLQueryExpr) left;
                            SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                            SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                            String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                            if (subSelectRet != null) {
                                return subSelectRet;
                            }
                        }
                        if (Objects.equals(right, left)) {
                            assert ret != null;
                            return ret.append("SELECT语句WHERE条件含有1 = 1！").toString();
                        }
                    }
                }
            }
        }
        return null;
    }

    /**
     * 校验新增语句
     *
     * @param sql 待校验语句
     * @return 返回信息
     */
    public static String parserCheckInsert(String sql) {
        StringBuilder ret = new StringBuilder();
        OracleStatementParser parser = new OracleStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();
        if (Objects.isNull(stmtList) || CollectionUtils.isEmpty(stmtList) || CollectionUtils.size(stmtList) >= STMTLISTSIZE) {
            ret.append("语句不正确！");
            return ret.toString();
        }
        for (SQLStatement stmt : stmtList) {
            if (stmt instanceof SQLInsertStatement) {
                SQLInsertStatement insertStatement = (SQLInsertStatement) stmt;
                // 如果含有SUB SELECT语句
                SQLSelect sqlSelect = insertStatement.getQuery();
                if (Objects.nonNull(sqlSelect)) {
                    SQLSelectQueryBlock queryBlock = sqlSelect.getQueryBlock();
                    String subSelectRet = parseSelectBlock(ret, queryBlock);
                    if (subSelectRet != null) {return subSelectRet;}
                }
            } else {
                ret.append("不是INSERT语句！");
            }
        }
        return ret.toString();
    }

    /**
     * 校验更新语句
     *
     * @param sql 待校验语句
     * @return 返回信息
     */
    public static String parserCheckUpdate(String sql) {
        StringBuilder ret = new StringBuilder();
        OracleStatementParser parser = new OracleStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();
        if (Objects.isNull(stmtList) || CollectionUtils.isEmpty(stmtList) || CollectionUtils.size(stmtList) >= STMTLISTSIZE) {
            ret.append("语句不正确！");
            return ret.toString();
        }
        for (SQLStatement stmt : stmtList) {
            if (stmt instanceof SQLUpdateStatement) {
                SQLUpdateStatement updateStatement = (SQLUpdateStatement) stmt;
                List<SQLUpdateSetItem> sqlUpdateSetItemList = updateStatement.getItems();
                if (Objects.isNull(sqlUpdateSetItemList) || CollectionUtils.isEmpty(sqlUpdateSetItemList)) {
                    ret.append("UPDATE语句没有SET！");
                }
                // SET 集合
                for (SQLUpdateSetItem sqlUpdateSetItem : sqlUpdateSetItemList) {
                    SQLExpr sqlUpdateExpr = sqlUpdateSetItem.getValue();
                    // SUB SELECT 语句
                    if (sqlUpdateExpr instanceof SQLQueryExpr) {
                        SQLQueryExpr sqlQueryExpr = (SQLQueryExpr) sqlUpdateExpr;
                        SQLSelect subSqlSelect = sqlQueryExpr.getSubQuery();
                        if (Objects.nonNull(subSqlSelect)) {
                            SQLSelectQueryBlock queryBlock = subSqlSelect.getQueryBlock();
                            String subSelectRet = parseSelectBlock(ret, queryBlock);
                            if (subSelectRet != null) {return subSelectRet;}
                        }
                    }
                }
                SQLExpr where = updateStatement.getWhere();
                if (Objects.isNull(where)) {
                    return ret.append("UPDATE语句没有WHERE条件！").toString();
                }

                if (where instanceof SQLBinaryOpExpr) {
                    SQLBinaryOpExpr sqlBinaryOpExpr = (SQLBinaryOpExpr) where;
                    SQLBinaryOperator sqlBinaryOperator = sqlBinaryOpExpr.getOperator();
                    if (Objects.equals(SQLBinaryOperator.Equality, sqlBinaryOperator)) {
                        SQLExpr right = sqlBinaryOpExpr.getRight();
                        if (right instanceof SQLQueryExpr) {
                            SQLQueryExpr itemQueryExpr = (SQLQueryExpr) right;
                            SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                            SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                            String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                            if (subSelectRet != null) {return subSelectRet;}
                        }
                        SQLExpr left = sqlBinaryOpExpr.getLeft();
                        if (left instanceof SQLQueryExpr) {
                            SQLQueryExpr itemQueryExpr = (SQLQueryExpr) left;
                            SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                            SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                            String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                            if (subSelectRet != null) {return subSelectRet;}
                        }
                        if (Objects.equals(right, left)) {
                            return ret.append("UPDATE语句WHERE条件含有1 = 1！").toString();
                        }
                    } else {
                        // WHERE条件集合
                        List<SQLObject> whereList = where.getChildren();
                        for (SQLObject so : whereList) {
                            if (so instanceof SQLBinaryOpExpr) {
                                SQLBinaryOpExpr sbo = (SQLBinaryOpExpr) so;
                                SQLExpr right = sbo.getRight();
                                if (right instanceof SQLQueryExpr) {
                                    SQLQueryExpr itemQueryExpr = (SQLQueryExpr) right;
                                    SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                                    SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                                    String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                                    if (subSelectRet != null) {
                                        return subSelectRet;
                                    }
                                }
                                SQLExpr left = sbo.getLeft();
                                if (left instanceof SQLQueryExpr) {
                                    SQLQueryExpr itemQueryExpr = (SQLQueryExpr) left;
                                    SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                                    SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                                    String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                                    if (subSelectRet != null) {
                                        return subSelectRet;
                                    }
                                }
                                if (Objects.equals(right, left)) {
                                    return ret.append("UPDATE语句WHERE条件含有1 = 1！").toString();
                                }
                            }
                        }
                    }
                }
            } else {
                ret.append("不是UPDATE语句！");
            }
        }
        return ret.toString();
    }

    /**
     * 校验删除语句
     *
     * @param sql 待校验语句
     * @return 返回信息
     */
    public static String parserCheckDelete(String sql) {
        StringBuilder ret = new StringBuilder();
        OracleStatementParser parser = new OracleStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();
        if (Objects.isNull(stmtList) || CollectionUtils.isEmpty(stmtList) || CollectionUtils.size(stmtList) >= STMTLISTSIZE) {
            ret.append("语句不正确！");
            return ret.toString();
        }
        for (SQLStatement stmt : stmtList) {
            if (stmt instanceof SQLDeleteStatement) {
                SQLDeleteStatement deleteStatement = (SQLDeleteStatement) stmt;
                SQLExpr where = deleteStatement.getWhere();
                if (Objects.isNull(where)) {
                    return ret.append("DELETE语句没有WHERE条件！").toString();
                }
                if (where instanceof SQLBinaryOpExpr) {
                    SQLBinaryOpExpr sqlBinaryOpExpr = (SQLBinaryOpExpr) where;
                    SQLBinaryOperator sqlBinaryOperator = sqlBinaryOpExpr.getOperator();
                    if (Objects.equals(SQLBinaryOperator.Equality, sqlBinaryOperator)) {
                        SQLExpr right = sqlBinaryOpExpr.getRight();
                        if (right instanceof SQLQueryExpr) {
                            SQLQueryExpr sqlWhereSubQueryExpr = (SQLQueryExpr) right;
                            SQLSelect sqlWhereSubSelect = sqlWhereSubQueryExpr.getSubQuery();
                            if (Objects.nonNull(sqlWhereSubSelect)) {
                                SQLSelectQueryBlock queryBlock = sqlWhereSubSelect.getQueryBlock();
                                String subSelectRet = parseSelectBlock(ret, queryBlock);
                                if (subSelectRet != null) {
                                    return subSelectRet;
                                }
                            }
                        }
                        SQLExpr left = sqlBinaryOpExpr.getLeft();
                        if (Objects.equals(right, left)) {
                            return ret.append("SELECT语句WHERE条件含有1 = 1！").toString();
                        }
                    } else {
                        // WHERE条件集合
                        List<SQLObject> whereList = where.getChildren();
                        for (SQLObject so : whereList) {
                            if (so instanceof SQLBinaryOpExpr) {
                                SQLBinaryOpExpr sbo = (SQLBinaryOpExpr) so;
                                SQLExpr right = sbo.getRight();
                                if (right instanceof SQLQueryExpr) {
                                    SQLQueryExpr itemQueryExpr = (SQLQueryExpr) right;
                                    SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                                    SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                                    String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                                    if (subSelectRet != null) {
                                        return subSelectRet;
                                    }
                                }
                                SQLExpr left = sbo.getLeft();
                                if (left instanceof SQLQueryExpr) {
                                    SQLQueryExpr itemQueryExpr = (SQLQueryExpr) left;
                                    SQLSelect itemSubSelect = itemQueryExpr.getSubQuery();
                                    SQLSelectQueryBlock subQueryBlock = itemSubSelect.getQueryBlock();
                                    String subSelectRet = parseSelectBlock(ret, subQueryBlock);
                                    if (subSelectRet != null) {
                                        return subSelectRet;
                                    }
                                }
                                if (Objects.equals(right, left)) {
                                    return ret.append("SELECT语句WHERE条件含有1 = 1！").toString();
                                }
                            }
                        }
                    }
                }
            } else {
                ret.append("不是DELETE语句！");
            }
        }
        return ret.toString();
    }

    /**
     * 校验DDL语句
     *
     * @param sql 待校验语句
     * @return 返回信息
     */
    public static String parserCheckDDL(String sql) {
        StringBuilder ret = new StringBuilder();
        OracleStatementParser parser = new OracleStatementParser(sql);
        List<SQLStatement> stmtList = parser.parseStatementList();
        if (Objects.isNull(stmtList) || CollectionUtils.isEmpty(stmtList) || CollectionUtils.size(stmtList) >= STMTLISTSIZE) {
            ret.append("语句不正确！");
            return ret.toString();
        }
        for (SQLStatement stmt : stmtList) {
            if (stmt instanceof SQLDDLStatement) {
                if (stmt instanceof SQLAlterTableStatement) {
                    // ignore
                } else if (stmt instanceof SQLCreateTableStatement) {
                    // ignore
                } else if (stmt instanceof SQLCreateSequenceStatement) {
                    // ignore
                } else if (stmt instanceof SQLCreateViewStatement) {
                    // ignore
                } else if (stmt instanceof SQLCreateIndexStatement) {
                    // ignore
                } else if (stmt instanceof SQLAlterTableAddColumn) {
                    // ignore
                } else if (stmt instanceof SQLAlterSequenceStatement) {
                    // ignore
                } else {
                    ret.append("不支持的语句！");
                }
            } else {
                ret.append("不支持的语句！");
            }
        }
        return ret.toString();
    }
}
